import 'dart:async';
import 'package:flutter/material.dart';
import 'package:nuntio_hera/hera_app/helpers.dart';
import 'package:nuntio_hera/hera_app/login_screen.dart';
import 'package:nuntio_core/nuntio_theme.dart';
import 'package:nuntio_core/models/hera.pb.dart';
import '../../hera/hera.dart';
import '../hera/models/auth.dart';
import 'hera_localizations.dart';

class HeraApp extends StatefulWidget {
  HeraApp({
    Key? key,
    required this.child,
    this.excludeAuthMechanisms,
    this.excludeOidcProviders,
    this.logo,
  });

  final List<LoginType>? excludeAuthMechanisms;
  final List<OidcProviderType>? excludeOidcProviders;
  final Widget child;
  final Widget? logo;

  // on authenticated go to child;
  @override
  State<HeraApp> createState() => _HeraAppState();
}

class _HeraAppState extends State<HeraApp> {
  late AuthState _authState;
  late Config _config;
  late TextInputType _inputType;

  Future<void> initialize() async {
    // get config used to login
    _config = await Hera.initializeAuth();
    if (await Hera.isAuthenticated()) {
      setState(() {
        _authState = AuthState.authenticated;
        _inputType = getInputType(_config.supportedLoginMechanisms);
      });
      return;
    }
    // do not display oidc proivers that are not enabled
    _config.oidcProviders.removeWhere((key, value) =>
        !value.enabled ||
        widget.excludeOidcProviders?.contains(value.type) == true);
    _config.supportedLoginMechanisms.removeWhere(
        (element) => widget.excludeAuthMechanisms?.contains(element) ?? false);
    setState(() {
      _inputType = getInputType(_config.supportedLoginMechanisms);
      _authState = AuthState.notAuthenticated;
    });
  }

  @override
  void initState() {
    Hera.onLogout = () {
      setState(() {
        _authState = AuthState.notAuthenticated;
      });
    };
    setState(() {
      _authState = AuthState.loading;
    });
    initialize();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      debugShowCheckedModeBanner: false,
      localizationsDelegates: [
        DefaultHeraLocalizations.delegate,
      ],
      theme: Theme.of(context).brightness == Brightness.light
          ? nuntioLightTheme
          : nuntioDarkTheme,
      home: Builder(
        builder: (context) {
          switch (_authState) {
            case AuthState.loading:
              return Center(
                child: CircularProgressIndicator(),
              );
            case AuthState.authenticated:
              return widget.child;
            case AuthState.notAuthenticated:
              return LoginScreen(
                inputType: _inputType,
                config: _config,
                onAuthenticated: (BuildContext context) {
                  initialize();
                },
              );
            default:
              return SizedBox(child: Text("error"));
          }
        },
      ),
    );
  }
}

class Test extends DefaultHeraLocalizations {
  @override
  String get next => "Forsæt";
}
